use Test::More tests => 49;
BEGIN { use_ok('Stanford::Certificate') };

use Stanford::Orange::Util qw/ trim slurp spew tempfile / ;
use Stanford::Certificate::Notification ;
use Data::Dumper ;

# Set the DEB_PACKAGE_BUILD environment variable.
require 't/run_non_build_tests.pl' ;

# Delete and re-create database.
require 't/setup_resetdb.pl' ;
resetdb() ;

my ($id, $request, $new_request, $cn) ;

# Setup schema and config
require 't/setup_schema.pl' ;
my ($schema, $CONFIG) = get_schema() ;

# Make a fake object
$request = Stanford::Certificate::make_fake_request($schema) ;

# Add a subject
$cn = 'testingabc.stanford.edu' ;
$request->cn('testingabc.stanford.edu') ;
$request->make_csr() ;

# Save it.
$request->saveme() ;
$id = $request->id() ;
ok ($id > 0) ;

# Read it.
$new_request = $schema->resultset('Request')->find($id);
ok ($new_request) ;
ok ($new_request->id() == $id) ;

# Change it.
$new_request->duration(2) ;
$new_request->saveme() ;

# Delete it.
$new_request->delete() ;
my $newer_request = $schema->resultset('Request')->find($id);
ok (!$newer_request) ;
$newer_request = $schema->resultset('Request')->find({'cn' => $cn,}) ;
ok (!$newer_request) ;

# Make a new one one, cancel it, and try to send.
$request = Stanford::Certificate::make_fake_request($schema) ;

$cn = 'testingdef.stanford.edu' ;
$request->cn($cn) ;
$request->make_csr() ;

# Save it.
$request->saveme() ;
$id = $request->id() ;
ok ($id > 0) ;

$request->cancel() ;
ok ($request->isCancelled()) ;
ok ($request->isCanceled()) ;
my $cm_credentials_yml_file = $CONFIG->incommon_cm_credentials_yaml_file() ;
eval
{
  $request->send($cm_credentials_yml_file) ;
} ;

ok ($@ =~ m{cancelled request}) ;

# Make a new one and try to make it fail some validations.
$request = Stanford::Certificate::make_fake_request($schema) ;
$cn = 'testingxyz.stanford.edu' ;
$request->cn($cn) ;
$request->make_csr() ;

# DURATION
$request->duration(undef) ;

eval
{
  $request->saveme() ;
} ;
ok ($@ =~ m{duration is not defined}) ;

$request->duration(10) ;
eval
{
  $request->saveme() ;
} ;
ok ($@ =~ m{duration must be}) ;
$request->duration(3) ;

# ## #### ## #### ## #### ## #### ## #### ## ###
my $test_empty_fref = sub
{
  my ($column) = @_ ;
  my $save_value = $request->get_column($column) ;

  $request->set_column($column => undef) ;
  eval
  {
    $request->saveme() ;
  } ;
  ok ($@) ;

  $request->set_column($column => q{}) ;
  eval
  {
    $request->saveme() ;
  } ;
  ok ($@) ;
  $request->set_column($column => $save_value) ;
} ;
# ## #### ## #### ## #### ## #### ## #### ## ###

$test_empty_fref->('cn') ;
$test_empty_fref->('requester_sunetid') ;
$test_empty_fref->('requester_email') ;
$test_empty_fref->('contact_email') ;
$test_empty_fref->('csr') ;

# Make sure we can use the request schema with the certificate object.
my $req_schema = $request->get_schema() ;
my $cert = $req_schema->resultset('Certificate')->new({});
my $pem_file1 = './t/incommon_root.pem' ;
my $new_cert1 = $cert->make_cert_from_pem_file($pem_file1) ;
ok ($new_cert1->id() > 0) ;
ok ($new_cert1->is_self_signed()) ;
ok ($new_cert1->issued_by_id() == $new_cert1->id()) ;
ok ($new_cert1->check_signing_chain()) ;

# Test expirations:
#  1. Add a request and associated expired cert and verify that this
#     request is returned
#  2. Add a second request with associated expired cert but with the SAME
#     subject as the first and verify that BOTH requests are returned but
#     that the one with the nearest expiration comes first.
use Date::Calc::Object ;
my ($request0, $request1, $dc, $dc_now) ;
my ($request_id0, $request_id1) ;
my ($mdm_cert0, $mdm_cert1) ; # $mdm_cert1 expires LATER than $mdm_cert0.
my @requests ;

$dc_now = Date::Calc->gmtime() ;

my $mdm_dev1_pem_file = './t/mdm-dev1.stanford.edu.pem' ;
my $mdm_dev2_pem_file = './t/mdm-dev2.stanford.edu.pem' ;
my $mdm_dev1_csr_file = './t/mdm-dev1.stanford.edu.csr' ;
my $mdm_dev2_csr_file = './t/mdm-dev2.stanford.edu.csr' ;

# # #### # #### # #### # #### # #### # #### # #### # #### # #### # ###
my $make_request_with_cert_fref = sub
{
  my ($dc_expires, $pem_file, $csr_file) = @_ ;
  my $request = Stanford::Certificate::make_fake_request($schema) ;

  # The 1 in the following call forces the certificate to be added
  # even if there is already a certificate with the same issuer and serial
  # number.
  my $mdm_cert = $cert->make_cert_from_pem_file($pem_file, 1) ;

  $mdm_cert->insert() ;
  $mdm_cert->discard_changes() ;
  $mdm_cert->expires_at(Stanford::Certificate::Notification::datecalc_to_mysql($dc_expires)) ;

  $mdm_cert->update() ;

  $request->cn($mdm_cert->cn()) ;
  $request->certificate_id($mdm_cert->id()) ;
  $request->csr(slurp($csr_file)) ;
  $request->insert() ;
  $request->discard_changes() ;

  return ($request, $mdm_cert) ;
} ;
# # #### # #### # #### # #### # #### # #### # #### # #### # #### # ###

# 1. Add a request and associated expired cert and verify that this
#    request is returned (24 and 48 hours)

# Make the cert expiration be 25 hours from now.
$dc = Date::Calc->gmtime() ;
$dc += [0,0,1,1,0,0] ;
($request0, $mdm_cert0)
  = $make_request_with_cert_fref->($dc, $mdm_dev1_pem_file, $mdm_dev1_csr_file) ;
$request_id0 = $request0->id() ;

# Change its expiration date to tomorrow.
@requests = $request->expiring_requests($dc_now, 1) ;
ok (scalar(@requests) == 1) ;
ok ($requests[0]->id() == $request_id0) ;

# suppress its notifications field
$request0->suppress_notifications(1) ;
$request0->update() ;
@requests = $request->expiring_requests($dc_now, 1) ;
ok (scalar(@requests) == 0) ;

# unsuppress the notifications field
$request0->suppress_notifications(0) ;
$request0->update() ;
@requests = $request->expiring_requests($dc_now, 1) ;
ok (scalar(@requests) == 1) ;
ok ($requests[0]->id() == $request_id0) ;

# Do it again, but this time make the cert expire 49 hours from now (so no
# requests should come back).
$dc = Date::Calc->gmtime() ;
$dc += [0,0,2,1,0,0] ;
$mdm_cert0->expires_at(Stanford::Certificate::Notification::datecalc_to_mysql($dc)) ;
$mdm_cert0->update() ;

@requests = $request->expiring_requests($dc_now, 1) ;
ok (scalar(@requests) == 0) ;

# Change range from (24, 48) hours to (24, 72) hours.
@requests = $request->expiring_requests($dc_now, 1, 3) ;
ok (scalar(@requests) == 1) ;

# Change range from (24, 72) hours to (48, 72) hours.
@requests = $request->expiring_requests($dc_now, 2, 3) ;
ok (scalar(@requests) == 1) ;

# Change expiration date back to 25 hours
$dc = Date::Calc->gmtime() ;

$dc += [0,0,1,1,0,0] ;
$mdm_cert0->expires_at(Stanford::Certificate::Notification::datecalc_to_mysql($dc)) ;
$mdm_cert0->update() ;


#  2. Add a second request with associated expired cert but with the SAME
#     subject as the first and verify that BOTH requests are returned but
#     that the one with the nearest expiration comes first. (48 and 72 hours)

# We want $mdm_cert1 to expire 27 hours from now, so a bit later than $mdm_cert0.
$dc = Date::Calc->gmtime() ;
$dc += [0,0,1,3,0,0] ;
($request1, $mdm_cert1)
  = $make_request_with_cert_fref->($dc, $mdm_dev1_pem_file, $mdm_dev1_csr_file) ;
$request_id1 = $request1->id() ;

# The number of requests expiring between 48 and 72 hours should now be two.
@requests = $request->expiring_requests($dc_now, 1) ;

ok (scalar(@requests) == 2) ;

# Verify that the FIRST request returned is $request1 (since it expires
# later).
ok ($requests[0]->id() == $request_id1) ;
ok ($requests[1]->id() == $request_id0) ;

#  3. We make things a bit trickier now. We add four requests:
#     R1. request for subject0 expires in 125 hours
#     R2. request for subject1 already expired
#     R3. request for subject0 expires in 1 year
#     R4. request for subject1 expires in 124 hours
#     R5. request for subject0 expires in 124 hours
#     R6. request for subject1 expires in 32 hours
#
# where subject0 is 'mdm-dev2' and subject1 is 'mdm-dev1'. We search for
# all requests that expire between 5 and 6 days (120 and 144 hours).
# The result _should_ be R4, R5, R1

my ($R1, $R2, $R3, $R4, $R5, $R6) ; # request objects
my ($C1, $C2, $C3, $C4, $C5, $C6) ; # certificate objects

# R1 (mdm-dev2 expires in 125 hours (5 days and 5 hours))
$dc = Date::Calc->gmtime() ;
$dc += [0,0,5,5,0,0] ;
($R1, $C1)
  = $make_request_with_cert_fref->($dc, $mdm_dev2_pem_file, $mdm_dev2_csr_file) ;

# R2 (mdm-dev1 expired 10 days ago)
$dc = Date::Calc->gmtime() ;
$dc += [0,0,-10,0,0,0] ;
($R2, $C2)
  = $make_request_with_cert_fref->($dc, $mdm_dev1_pem_file, $mdm_dev1_csr_file) ;

# R3 (mdm-dev2 expires in one year)
$dc = Date::Calc->gmtime() ;
$dc += [1,0,0,0,0,0] ;
($R3, $C3)
  = $make_request_with_cert_fref->($dc, $mdm_dev2_pem_file, $mdm_dev2_csr_file) ;

# R4 (mdm-dev1 expires in 124 hours (5 days and 4 hours))
$dc = Date::Calc->gmtime() ;
$dc += [0,0,5,4,0,0] ;
($R4, $C4)
  = $make_request_with_cert_fref->($dc, $mdm_dev1_pem_file, $mdm_dev1_csr_file) ;

# R5 (mdm-dev2 expires in 124 hours (5 days and 4 hours))
$dc = Date::Calc->gmtime() ;
$dc += [0,0,5,4,0,0] ;
($R5, $C5)
  = $make_request_with_cert_fref->($dc, $mdm_dev2_pem_file, $mdm_dev2_csr_file) ;

# R6 (mdm-dev1 expires in 32 hours (1 day and 8 hours))
$dc = Date::Calc->gmtime() ;
$dc += [0,0,1,8,0,0] ;
($R6, $C6)
  = $make_request_with_cert_fref->($dc, $mdm_dev1_pem_file, $mdm_dev1_csr_file) ;

# The number of requests expiring between 120 and 144 hours (5 and 6
# days) should now be three and in this order: R4, R1, R5
@requests = $request->expiring_requests($dc_now, 5, 6) ;
ok (scalar(@requests) == 3) ;

ok ($requests[0]->id() == $R4->id()) ;
ok ($requests[1]->id() == $R1->id()) ;
ok ($requests[2]->id() == $R5->id()) ;

# Remove the duplicate requests. That is, leave only one request for each
# CN, that request with the later (or latest) expiration date. In this
# test, that means we should end up with certificates R4 and R1, as R5 and
# R1 have the same CN but R1 has the later expiration.
@requests = Stanford::Certificate::Notification::unique_latest_requests(\@requests) ;

ok (scalar(@requests) == 2) ;
ok ($requests[0]->id() == $R4->id()) ;
ok ($requests[1]->id() == $R1->id()) ;

# Given a list of requests that are soon to expire, remove any that have a
# CN where there is another request with a certiicate with the same CN but
# a _later_ expiration date.

# We next test the requests_with_no_later function. This function removes
# any requests whose common names belong to _other_ certificates that are
# expiring later than the certificate associated with the request.
#
# For this test we need two certificates with different serial numbers but
# the same common name.

# Delete all the certificates and requests (from t/setup_resetdb.pl):
delete_all_certs_and_requests($schema) ;

my ($req_renewed0,  $req_renewed1) ;
my ($cert_renewed0, $cert_renewed1) ;

my $renewed0_pem_file = './t/renewed01.pem' ;
my $renewed1_pem_file = './t/renewed02.pem' ;
my $renewed0_csr_file = './t/renewed01.csr' ;
my $renewed1_csr_file = './t/renewed02.csr' ;

# We want cert0 to expire in 25 hours and cert1 to expire in 52
# hours. This way, when we do the filtering, we should be left with nothing!
$dc = Date::Calc->gmtime() ;
$dc += [0,0,1,1,0,0] ;
($req_renewed0, $cert_renewed0)
  = $make_request_with_cert_fref->($dc, $renewed0_pem_file, $renewed0_csr_file) ;

$dc = Date::Calc->gmtime() ;
$dc += [0,0,2,4,0,0] ;
($req_renewed1, $cert_renewed1)
  = $make_request_with_cert_fref->($dc, $renewed1_pem_file, $renewed1_csr_file) ;

@requests = $request->expiring_requests($dc_now, 1) ;
ok (scalar(@requests) == 1) ;

# The unique_latest_requests filter shouldn't change anything.
@requests = Stanford::Certificate::Notification::unique_latest_requests(\@requests) ;
ok (scalar(@requests) == 1) ;

# The requests_with_no_later function should remove all the requests
@requests = Stanford::Certificate::Notification::requests_with_no_later(\@requests) ;
ok (scalar(@requests) == 0) ;

# What if we pass in the empty array?
@requests = Stanford::Certificate::Notification::requests_with_no_later([ ]) ;
ok (scalar(@requests) == 0) ;
@requests = Stanford::Certificate::Notification::unique_latest_requests([ ]) ;
ok (scalar(@requests) == 0) ;

# Send an e-mail (but not when building package)
SKIP:
{
  if (defined($ENV{'DEB_PACKAGE_BUILD'}) && (defined($ENV{'DEB_PACKAGE_BUILD'})eq '1'))
  {
    skip "skipping e-mail send", 0 ;
  }

  $dc = Date::Calc->gmtime() ;
  $dc += [0,10,2,1,0,0] ;
  ($request0, $mdm_cert0)
    = $make_request_with_cert_fref->($dc, $mdm_dev1_pem_file, $mdm_dev1_csr_file) ;

  $request0->contact_email('adamhl@stanford.edu') ;
  $request0->send_expiration_warning_email() ;
}

# Cleanup.
$request0->delete() ;
