use Test::More tests => 25 ;

BEGIN { use_ok('Stanford::Certificate') };

use Data::Dumper ;
use Stanford::Orange::Util qw/ slurp / ;

# Set the DEB_PACKAGE_BUILD environment variable.
require 't/run_non_build_tests.pl' ;

# Setup schema and config
require 't/setup_schema.pl' ;
my ($schema, $CONFIG) = get_schema() ;

# Create an InCommon status.
my $incommon_status = $schema->resultset('IncommonStatus')->new({});
ok ($incommon_status) ;

# Get a certificate.
my $cert = $schema->resultset('Certificate')->new({});
ok ($cert) ;
my $pem_file1 = './t/incommon_root.pem' ;
my $new_cert1 = $cert->make_cert_from_pem_file($pem_file1) ;
ok ($new_cert1->id() > 0) ;

# Make a status for this cert.
$new_cert1->add_to_incommon_statuses({'status' => 'issued'}) ;
my @incommon_statuses = $new_cert1->incommon_statuses() ;
ok (@incommon_statuses) ;
ok (scalar(@incommon_statuses) == 1) ;

my $incommon_status1 = $incommon_statuses[0] ;
ok ($incommon_status1->is_issued()) ;
ok (!$incommon_status1->is_expired()) ;
ok (!$incommon_status1->is_declined()) ;
ok (!$incommon_status1->is_revoked()) ;

$new_cert1->update() ;

# Try to add another issued status.
eval {
     $new_cert1->add_to_incommon_statuses({'status' => 'issued'}) ;
} ;
if ($@) {
    ok(1) ;
} else {
    ok (0, 'failed to throw exception when unique constriant violated') ;
}

# Add a different status
sleep(2) ;
$new_cert1->add_to_incommon_statuses({'status'   => 'revoked',
                                      'status_at'=> '2014-0404 15:23:43'}) ;
$new_cert1->update() ;

@incommon_statuses = $new_cert1->incommon_statuses() ;
ok (scalar(@incommon_statuses) == 2) ;

# Add a different status
my $incommon_status_approved = $schema->resultset('IncommonStatus')->new({});
$incommon_status_approved->status('approved') ;
$new_cert1->add_status($incommon_status_approved) ;
$new_cert1->update() ;
@incommon_statuses = $new_cert1->incommon_statuses() ;
ok (scalar(@incommon_statuses) == 3) ;
# warn $new_cert1->to_s() ;

# Sort the statuses.
my @sorted =
  Stanford::Certificate::Result::IncommonStatus::sort_statuses(\@incommon_statuses) ;

ok ($sorted[0]->is_revoked()) ;
ok ($sorted[1]->is_issued()) ;
ok ($sorted[2]->is_approved()) ;

$incommon_status = $new_cert1->status() ;
ok ($incommon_status->is_revoked()) ;

# Test with cert having NO incommon statuses.
my $pem_file2 = './t/multi_domain.pem' ;
my $new_cert2 = $cert->make_cert_from_pem_file($pem_file2) ;
ok (!defined($new_cert2->status())) ;

#
my $status_json1 = qq/{"status": "Expired", "requested": "2011-05-12 09:24:28.000024", "serverType": "OTHER", "orderNumber": "-1", "expires": "2013-01-20 15:59:59", "commonName": "win-sts.stanford.edu", "type": "Unmanaged", "id": "16739", "subject": "www.win-sts.stanford.edu"}/;

my $id ;
($order_id, @incommon_statuses) = $incommon_status->parse_json($status_json1) ;
ok ($order_id == 16739) ;
ok (scalar(@incommon_statuses) == 2) ;

foreach my $incommon_status (@incommon_statuses) {
    $new_cert2->add_status($incommon_status) ;
}
$new_cert2->update() ;
ok ($new_cert2->status()->is_expired()) ;

## In this section we parse a json file of statuses

# Make a request and set its order number to one of the elements of the
# JSON file that was DECLINED.
my $request = $schema->resultset('Request')->new({});

my $request1 = Stanford::Certificate::make_fake_request($schema) ;
$request1->cn('filemaker.stanford.edu') ;
$request1->order_number('314081') ;
$request1->csr('XXX') ;
$request1->saveme() ;
ok (!$request1->is_declined()) ;

# Make a request and set its order number to one of the elements of the
# JSON file that was REJECTED.
my $request2 = Stanford::Certificate::make_fake_request($schema) ;
$request2->cn('weblogin-dev.stanford.edu') ;
$request2->order_number('19894') ;
$request2->csr('XXX') ;
$request2->saveme() ;
ok (!$request2->is_rejected()) ;

#warn $request1->to_s() ;
#warn $request2->to_s() ;

# Parse the JSON status file
Stanford::Certificate::Misc::load_incommon_statuses($schema,
  './t/incommon.json', 'quiet') ;

$request1 = $request->findByOrderNumber('314081') ;
$request2 = $request->findByOrderNumber('19894') ;

ok ($request1->is_declined()) ;
ok ($request2->is_rejected()) ;


