# Use: python table.py platform
#
# where platform is one of:
#  - testing (for Perl tests)
#  - sbx (Adam's development environment)
#  - uat (for tools-dev)

import sys

# Get the first argument
if (len(sys.argv) > 1):
  platform = sys.argv[1]
else:
  raise Exception("You must supply a platform")

# Anything that looks like "test" is the testing platform.
import re
if (re.search('test', platform)):
  platform = 'testing'

#print "platform is " + platform
#sys.exit(0)

from sqlalchemy import *
import yaml
import os

# Load YAML connection file
platform_to_file = {
  'testing': os.path.dirname(os.path.abspath(__file__)) + '/../sslcert_db_testing.yml',
  'uat': os.path.dirname(os.path.abspath(__file__)) + '/../sslcert_db_uat.yml',
  'sbx': os.path.dirname(os.path.abspath(__file__)) + '/../sslcert_db_sbx.yml',
#  'production': os.path.dirname(os.path.abspath(__file__)) + '/../sslcert_db_prod.yml',
}

connection_file_default = '/etc/certreq/cert-request-db.yml'

# If the platform is not in the platform_to_file mapping
# use the default
if (platform in platform_to_file):
  connection_file = platform_to_file[platform]
else:
  connection_file = connection_file_default

if not os.path.isfile(connection_file):
  raise Exception("could not find connection file " + connection_file)

# stream = file(os.path.dirname(os.path.abspath(__file__)) + '/../connection.yml', 'r')
stream = file(connection_file, 'r')
connection_yml = yaml.load(stream)
connection = connection_yml

# Connect to the database
dsn = connection['python_dsn']
engine = create_engine(dsn)

metadata = MetaData()

if (platform != 'testing'):
  print "WARNING!!! This script will DELETE ALL DATA in the database."
  print "WARNING!!! Run this ONLY when initially setting up database."
  print "WARNING!!! Platform --> " + platform
  var = raw_input("Hit enter to continue, Ctrl-C to abort.")


server_types = Table('server_types', metadata,
    Column('id',                 Integer, primary_key = True),
    Column('name',               String(60), nullable = False, unique = True),
    Column('incommonid',         Integer, nullable = False, unique = True),
    Column('created_at',         DateTime, nullable = False),
    Column('updated_at',         DateTime, nullable = False),
    mysql_engine='InnoDB',
)

certificate_types = Table('certificate_types', metadata,
    Column('id',                 Integer, primary_key = True),
    Column('name',               String(60), nullable = False, unique = True),
    Column('description',        Text, nullable = False),
    Column('created_at',         DateTime, nullable = False),
    Column('updated_at',         DateTime, nullable = False),
    mysql_engine='InnoDB',
)

requests = Table('requests', metadata,
    Column('id',                 Integer, primary_key = True),
    Column('cn',                 String(100), nullable = False),
    Column('cancelled',          Boolean, nullable = False, default = False),
    Column('declined',           Boolean, nullable = False, default = False),
    Column('rejected',           Boolean, nullable = False, default = False),
    Column('test',               Boolean, nullable = False, default = False),
    Column('requester_sunetid',  String(12), nullable = False),
    Column('requester_fullname', String(60), nullable = True),
    Column('requester_email'  ,  String(60), nullable = False),
    Column('requester_affiliation', String(30), nullable = True),
    Column('requester_department',  String(90), nullable = True),
    Column('requester_comments',     Text, nullable = True),
    Column('requester_alternatives', Text, nullable = True),
    Column('server_type_id',     Integer, ForeignKey("server_types.id"), nullable = False),
    Column('contact_email',      String(60), nullable = False),
    Column('csr',                Text, nullable = False),
    Column('reissue',            Boolean, nullable = False, default = False),
    Column('duration',           Integer, nullable = False),
    Column('alternatives',       Text, nullable = True),
    Column('order_number',       String(16),  nullable = True),
    Column('approved',           Boolean,  nullable = False, default = False),
    Column('fulfilled',          Boolean,  nullable = False, default = False),
    Column('certificate_id',     Integer, ForeignKey("certificates.id"), nullable = True),
    Column('approval_contacts',  String(100),  nullable = False),
    Column('renewid',            String(30),  nullable = True),
    Column('certificate_type_id', Integer, ForeignKey("certificate_types.id"), nullable = False),
    Column('incommon_cert_number', Integer,   nullable = True),
    Column('incommon_cert_name', String(60),  nullable = True),
    Column('notes',              Text,     nullable = True),
    Column('suppress_notifications', Boolean,  nullable = False, default = False),
    Column('created_at',         DateTime, nullable = False),
    Column('updated_at',         DateTime, nullable = False),
    mysql_engine='InnoDB',
)

certificates = Table('certificates', metadata,
    Column('id',                 Integer, primary_key = True),
    Column('version',            Integer, nullable = False),
    Column('subject',            String(500), nullable = False),
    Column('issuer',             String(500), nullable = False),
    Column('cn',                 String(100), nullable = False),
    Column('valid_at',           DateTime, nullable = False),
    Column('expires_at',         DateTime, nullable = False),
    Column('issued_by_id',       Integer, ForeignKey("certificates.id"), nullable = True),
    Column('pem',                Text,        nullable = False),
    Column('serial_number',      String(60),  nullable = True),
    Column('signature_algorithm',String(30),  nullable = True),
    Column('key_algorithm',      String(15),  nullable = False),
    Column('key_length',         Integer,  nullable = True),
    Column('subject_key_identifier',   String(60),  nullable = True),
    Column('authority_key_identifier', String(60),  nullable = True),
    Column('ca',                 Boolean, nullable = False, default = False),
    Column('key_cert_sign',      Boolean, nullable = False, default = False),
    Column('crl_url',            String(120), nullable = True),
    Column('ocsp_uri',           String(120), nullable = True),
    Column('contact_email',      String(60), nullable = True),
    Column('suppress_notifications', Boolean,  nullable = False, default = False),
    Column('created_at',         DateTime,   nullable = False),
    Column('updated_at',         DateTime,   nullable = False),
    mysql_engine='InnoDB',
)

san_types = Table('san_types', metadata,
    Column('id',                 Integer,    primary_key = True),
    Column('name',               String(30), nullable = False, unique = True),
    Column('created_at',         DateTime,   nullable = False),
    Column('updated_at',         DateTime,   nullable = False),
    mysql_engine='InnoDB',
)

sans = Table('sans', metadata,
    Column('id',                 Integer,    primary_key = True),
    Column('certificate_id',     Integer,    ForeignKey("certificates.id"), nullable = False),
    Column('name',               String(60), nullable = False),
    Column('san_type_id',        Integer,    ForeignKey("san_types.id"), nullable = False),
    Column('created_at',         DateTime,   nullable = False),
    Column('updated_at',         DateTime,   nullable = False),
    mysql_engine='InnoDB',
)

eventlog = Table('eventlog', metadata,
    Column('id',                 Integer, primary_key = True),
    Column('request_id',         Integer, ForeignKey("requests.id"), nullable = True),
    Column('certificate_id',     Integer, ForeignKey("certificates.id"), nullable = True),
    Column('level',              Integer, nullable = False),
    Column('message',            Text, nullable = False),
    Column('created_at',         DateTime, nullable = False),
    mysql_engine='InnoDB',
)

legacy_expirations = Table('legacy_expirations', metadata,
    Column('id',                 Integer,    primary_key = True),
    Column('contact_email',      String(60), nullable = False),
    Column('cn',                 String(60), nullable = False),
    Column('expires_at',         Date, nullable = False),
    Column('suppress_notifications', Boolean,  nullable = False, default = False),
    Column('created_at',         DateTime,   nullable = False),
    Column('updated_at',         DateTime,   nullable = False),
    mysql_engine='InnoDB',
)

incommon_statuses = Table('incommon_statuses', metadata,
    Column('id',                 Integer, primary_key = True),
    Column('certificate_id',     Integer, ForeignKey("certificates.id"), nullable = True),
    Column('status',             Enum('approved', 'applied', 'declined',
                                      'expired', 'invalid', 'issued', 'rejected',
                                      'requested', 'revoked', 'unmanaged'), nullable = False),
    Column('status_at',          DateTime, nullable = True),
    Column('created_at',         DateTime, nullable = False),
    mysql_engine='InnoDB',
)
Index('cert_id_status_uq_idx', incommon_statuses.c.certificate_id, incommon_statuses.c.status,
      unique = True)


metadata.drop_all(engine)
metadata.create_all(engine)


